pub const AUTHORS: &str = env!("CARGO_PKG_AUTHORS");
pub const VERSION: &str = env!("CARGO_PKG_VERSION");
pub const BUILDTD: &str = env!("BUILD_TIMEDATE");
pub const REVISION: &str = env!("VCS_COMMIT_HASH");
pub const REVDATE: &str = env!("VCS_COMMIT_DATE");

use knockout::{self, Thread};

use std::{collections::HashMap, env};

#[derive(Debug, Clone)]
struct HighestRating {
    count: u64,
    thread_id: u64,
    page_id: u64,
    post_id: u64,
}

#[derive(Debug, Clone, Hash, PartialEq, Eq, PartialOrd, Ord)]
struct HashRating {
    id: u64,
    name: String,
}

fn main() -> Result<(), eyre::Error> {
    #[cfg(target_os = "windows")]
    {
        ansi_term::enable_ansi_support()
            .map_err(|e| eyre::Report::msg(format!("Cannot enable ANSI color on Windows: {}", e)))?
    };
    let dir = std::fs::read_dir("threads/")?;
    let threads = dir
        .into_iter()
        .filter_map(|f| f.ok())
        .map(|f| f.path())
        .filter(|p| !p.to_string_lossy().ends_with(".err.json"))
        .map(|p| std::fs::read_to_string(&p))
        .filter_map(|f| {
            // Ugly hack because .inspect_err() is unstable.
            let _ = f.as_ref().map_err(|e| {
                println!("Unable to read reason:{}", e);
                e
            });
            f.ok()
        });
    let mut ratings: HashMap<HashRating, HighestRating> = HashMap::new();

    for data in threads {
        let thread: Thread = match serde_json::from_str(&data) {
            Ok(v) => v,
            Err(e) => {
                println!("Unable to deser reason:{}", e);
                continue;
            }
        };
        println!("Processing thread: {}", thread.id);
        for post in thread.posts.into_iter() {
            for rating in post.ratings.into_iter() {
                let rat = HashRating {
                    id: rating.ratingId,
                    name: rating.rating,
                };
                match ratings.get_mut(&rat) {
                    Some(v) => {
                        if rating.count > v.count {
                            v.count = rating.count;
                            v.thread_id = thread.id;
                            v.page_id = post.page;
                            v.post_id = post.id;
                        }
                    }
                    None => {
                        ratings.insert(
                            rat,
                            HighestRating {
                                count: rating.count,
                                thread_id: thread.id,
                                page_id: post.page,
                                post_id: post.id,
                            },
                        );
                    }
                };
            }
        }
    }
    let mut sorted: Vec<(HashRating, HighestRating)> = ratings.into_iter().collect();
    sorted.sort_by(|l, r| l.1.count.cmp(&r.1.count));
    println!("Highest Rated:");
    for rat in sorted {
        println!(
            "{}\t({}):\thttps://knockout.chat/thread/{}/{}/#post-{}",
            rat.0.name, rat.1.count, rat.1.thread_id, rat.1.page_id, rat.1.post_id
        )
    }
    Ok(())
}
